/*
 * Copyright (c) 2002, 2003, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbget2int.c,v 1.9 2005/03/29 19:53:12 ca Exp $")

#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/limits.h"
#include "sm/rcb.h"
#include "sm/reccom.h"

/*
**  SM_RCB_GET2UINT32 -- Read two ints from the current position in a sm_rcb_P.
**
**	Parameters:
**		rcb -- sm_rcb_P object to read int from.
**		pv1 -- pointer to value to return.
**		pv2 -- pointer to value to return.
**
**	Returns:
**		usual sm_error code; SM_E_OVFLW_NS
**
**	Last code review: 2005-03-29 05:29:06
**	Last code change:
*/

sm_ret_T
sm_rcb_get2uint32(sm_rcb_P rcb, uint32_t *pv1, uint32_t *pv2)
{
	SM_IS_RCB(rcb);
#if SM_RCB_CHECK
	SM_REQUIRE(rcb->sm_rcb_state == SM_RCB_DEC);
#endif
#if PARANOID
	SM_REQUIRE(pv1 != NULL);
	SM_REQUIRE(pv2 != NULL);
	SM_REQUIRE(rcb->sm_rcb_base != NULL);
#endif
	if (rcb->sm_rcb_len < rcb->sm_rcb_rw + sizeof(uint32_t) * 2)
		return sm_error_perm(SM_EM_RECCOM, SM_E_OVFLW_NS);
	sm_buf2uint32(rcb->sm_rcb_base + rcb->sm_rcb_rw, pv1);
	rcb->sm_rcb_rw += sizeof(uint32_t);
	sm_buf2uint32(rcb->sm_rcb_base + rcb->sm_rcb_rw, pv2);
	rcb->sm_rcb_rw += sizeof(uint32_t);
	return SM_SUCCESS;
}
